import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

class CustomShowModalBottomSheet {
  static Future<T?> customShowModalBottomSheet<T>({
    required BuildContext context,
    required WidgetBuilder builder,
    bool? dismissible,
    required bool darkMode,
  }) {
    assert(debugCheckHasMaterialLocalizations(context));
    return Navigator.push(
        context,
        _CullModalBottomSheetRoute<T>(
          builder: builder,
          theme: Theme.of(context),
          darkMode: darkMode,
          barrierLabel:
              MaterialLocalizations.of(context).modalBarrierDismissLabel,
          dismissible: dismissible ?? true,
        ));
  }
}

class _CullModalBottomSheetRoute<T> extends PopupRoute<T> {
  _CullModalBottomSheetRoute({
    this.darkMode,
    this.builder,
    this.theme,
    this.barrierLabel,
    this.dismissible,
    RouteSettings? settings,
  }) : super(settings: settings);

  final WidgetBuilder? builder;
  final ThemeData? theme;
  final bool? dismissible;
  final bool? darkMode;

  @override
  Duration get transitionDuration => const Duration(milliseconds: 200);

  @override
  bool get barrierDismissible => dismissible != null ? dismissible! : true;

  @override
  final String? barrierLabel;

  @override
  Color get barrierColor => Colors.black54;

  AnimationController? _animationController;

  @override
  AnimationController createAnimationController() {
    assert(_animationController == null);
    _animationController =
        BottomSheet.createAnimationController(navigator!.overlay!);
    return _animationController!;
  }

  @override
  Widget buildPage(BuildContext context, Animation<double> animation,
      Animation<double> secondaryAnimation) {
    Widget bottomSheet = MediaQuery.removePadding(
      context: context,
      removeTop: true,
      child: _CustomModalBottomSheet<T>(
        route: this,
        canDismiss: dismissible ?? false,
      ),
    );
    if (theme != null) bottomSheet = Theme(data: theme!, child: bottomSheet);
    return bottomSheet;
  }
}

class _CustomModalBottomSheet<T> extends StatefulWidget {
  const _CustomModalBottomSheet({Key? key, this.route, this.canDismiss = true})
      : super(key: key);

  final _CullModalBottomSheetRoute<T>? route;
  final bool canDismiss;

  @override
  _CustomModalBottomSheetState<T> createState() =>
      _CustomModalBottomSheetState<T>();
}

class _CustomModalBottomSheetState<T>
    extends State<_CustomModalBottomSheet<T>> {
  @override
  Widget build(BuildContext context) {
    final MediaQueryData mediaQuery = MediaQuery.of(context);
    final MaterialLocalizations localizations =
        MaterialLocalizations.of(context);
    String? routeLabel;
    switch (defaultTargetPlatform) {
      case TargetPlatform.iOS:
        routeLabel = '';
        break;
      case TargetPlatform.android:
      case TargetPlatform.linux:
      case TargetPlatform.macOS:
      case TargetPlatform.windows:
      case TargetPlatform.fuchsia:
        routeLabel = localizations.dialogLabel;
        break;
    }

    return Theme(
      data: Theme.of(context).copyWith(canvasColor: Colors.transparent),
      child: AnimatedBuilder(
          animation: widget.route!.animation!,
          builder: (BuildContext context, Widget? child) {
            // Disable the initial animation when accessible navigation is on so
            // that the semantics are added to the tree at the correct time.
            final double animationValue = mediaQuery.accessibleNavigation
                ? 1.0
                : widget.route!.animation!.value;
            return Semantics(
              scopesRoute: true,
              namesRoute: true,
              label: routeLabel,
              explicitChildNodes: true,
              child: ClipRect(
                child: CustomSingleChildLayout(
                  delegate: _CustomModalBottomSheetLayout(animationValue),
                  child: BottomSheet(
                    enableDrag: widget.canDismiss,
                    animationController: widget.route!._animationController,
                    onClosing: () => Navigator.pop(context),
                    builder: widget.route!.builder!,
                  ),
                ),
              ),
            );
          }),
    );
  }
}

class _CustomModalBottomSheetLayout extends SingleChildLayoutDelegate {
  _CustomModalBottomSheetLayout(this.progress);

  final double progress;

  @override
  BoxConstraints getConstraintsForChild(BoxConstraints constraints) {
    return BoxConstraints(
        minWidth: constraints.maxWidth,
        maxWidth: constraints.maxWidth,
        minHeight: 0.0,
        maxHeight: constraints.maxHeight);
  }

  @override
  Offset getPositionForChild(Size size, Size childSize) {
    return Offset(0.0, size.height - childSize.height * progress);
  }

  @override
  bool shouldRelayout(_CustomModalBottomSheetLayout oldDelegate) {
    return progress != oldDelegate.progress;
  }
}
